### -*-awk-*-
### ====================================================================
###  @Awk-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "0.07",
###     date            = "16 September 2013",
###     time            = "07:27:15 MDT",
###     filename        = "make-rc-file.awk",
###     address         = "University of Utah
###                        Department of Mathematics, 110 LCB
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "35778 111 516 4470",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "cattobib initialization files",
###     license         = "public domain",
###     supported       = "yes",
###     docstring       = "Filter one or more cattobib initialization
###                        files given on the command line to produce
###                        a new [usually temporary] initialization file
###                        devoid of comments, and in which all aliases
###                        in definitions have been expanded to their
###                        Z39.50 library catalog values.
###
###                        The order of processing each input line is:
###
###                            (1) strip all comments (text from sharp
###                                to end of line;
###
###                            (2) while a line ends in backslash, drop
###                                that backslash and append the next
###                                line, if any, after stripping comments
###                                from it
###
###                            (3) if the line looks like
###
###                                    alias name
###
###                                (re)define name to an empty string
###
###                            (4) if the line looks like
###
###                                    alias name v1 v2 v3 ...
###
###                                save the alias name and its value
###                                AFTER substituting expansions of
###                                any of the value words that are
###                                themselves aliases, and output the
###                                line to stdout
###
###                            (5) otherwise, discard the line
###
###                        That ensures that comments cannot continue
###                        beyond the current line, and the usual
###                        convention of collapsing backslash-newline
###                        to an empty string is enforced.  There is
###                        no provision for escaping sharp: it ALWAYS
###                        starts a comment.  That is in practice not
###                        a limitation, because Z39.50 addresses do
###                        not require that character.
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

	{ sub("#.*$", " ") }		# (1) strip comments

/^[ \t]*alias[ \t]/ && (NF >= 2) \
	{
	    line = $0

	    ## join continuation lines
	    while (line ~ "\\\\$")
	    {
		getline more
		sub("#.*$", " ", more)
		line = (substr(line, 1, length(line) - 1) more)
	    }

	    n = split(line, words)

	    if (n == 2)
	    {
		Alias[words[2]] = ""	# undefine alias
		printf("%s\t%s\n", words[1], words[2])
	    }
	    else if (n >= 3)		# expand alias values and output a new alias
	    {
		value = ""

		for (k = 3; k <= n; ++k)
		    value = (value " " ((words[k] in Alias) ? Alias[words[k]] : words[k]))

		sub("^[ \t]+", "",  value)
		sub("[ \t]+$", "",  value)
		gsub("[ \t]+", " ", value)
		Alias[words[2]] = value
		printf("%s\t%-15s\t%s\n", words[1], words[2], Alias[words[2]])
	    }
	}
